# -*- coding: utf-8 -*-
import unittest

import math

from dewloosh.core.tools.tools import float_to_str_sig, issequence, \
    suppress, timeit
from dewloosh.core.tools.alphabet import alphabet, ordrange, latinrange, \
    urange, greekrange, arabicrange
from dewloosh.core.tools.kwargtools import isinkwargs, allinkwargs, anyinkwargs, \
    getfromkwargs, popfromkwargs, getallfromkwargs, getasany, countkwargs


class TestTools(unittest.TestCase):

    def test_tools(self):
        assert float_to_str_sig(math.pi, sig=4) == '3.142'
        seq = [math.pi, math.pi]
        assert issequence(seq)
        assert not issequence(seq[0])
        assert float_to_str_sig(seq, sig=4) == ['3.142', '3.142']
        assert issequence([1, 2, 3])
        assert not issequence('123')

    def test_kwargtools(self):
        kwargs = dict(a = 1, c = 2)
        assert isinkwargs('a', **kwargs)
        assert all(isinkwargs(['a', 'c'], **kwargs))
        assert not isinkwargs('b', **kwargs)
        assert not any(isinkwargs(['b', 'd'], **kwargs))
        assert any(isinkwargs(['a', 'b'], **kwargs))
        assert any(isinkwargs(['c', 'd'], **kwargs))
        assert allinkwargs(['a', 'c'], **kwargs)
        assert allinkwargs('a', **kwargs)
        assert anyinkwargs(['d', 'c'], **kwargs)
        assert anyinkwargs('c', **kwargs)
        assert getfromkwargs(['a'], None, int, **kwargs) == [1]
        assert getfromkwargs(['b'], None, None, **kwargs) == [None]
        assert getallfromkwargs(['a', 'c'], None, **kwargs) == [1, 2]
        assert getasany(['a', 'b'], None, **kwargs) == 1
        
        d = {'E1': 1, 'E2': 2, 'G12': 12, 'NU23': 0}
        nE = countkwargs(lambda s: s[0] == 'E', **d)
        nG = countkwargs(lambda s: s[0] == 'G', **d)
        nNU = countkwargs(lambda s: s[0:2] == 'NU', **d)
        assert nE == 2
        assert nG == 1
        assert nNU == 1
        popfromkwargs(['E1'], d)
        assert isinstance(popfromkwargs(['E2'], d, astype=float)[0], float)
        assert 'E1' not in d
        
    def test_alphabet(self):
        for abctype in ['ord', 'latin', 'u', 'greek']:
            g = alphabet(abctype)
            [next(g) for i in range(5)]
        
        ordrange(5)
        latinrange(5)
        urange(5)
        greekrange(5)
        arabicrange(5)
                
        lrange = latinrange(5, start='i')
        grange = greekrange(5)
        orange = ordrange(5)
        arange = arabicrange(5, start=1)

        abc = alphabet('latin', start='i')
        next(abc)

        abc = alphabet(start='u')
        next(abc)
            
        abc = alphabet('u', start='\x03')
        next(abc)
        
        abc = alphabet('u', start='\x03')
        pokerstr = [next(abc) for _ in range(4)]
        
        
    def test_misc(self):
        
        @timeit
        def foo(): return None
        
        @suppress
        def foo(): return None
            


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from typing import Callable
import numpy as np
from numpy import ndarray


__all__ = ['squeeze']


def squeeze_if_array(arr):
    return np.squeeze(arr) if isinstance(arr, ndarray) else arr


def squeeze(default=True):
    def decorator(fnc: Callable):
        def inner(*args, squeeze:bool=default, **kwargs):
            if squeeze:
                res = fnc(*args, **kwargs)
                if isinstance(res, tuple):
                    return list(map(squeeze_if_array, res))
                elif isinstance(res, dict):
                    return {k : squeeze_if_array(v) for k, v in res.items()}
                else:
                    return squeeze_if_array(res)
            else:
                return fnc(*args, **kwargs)
        inner.__doc__ = fnc.__doc__
        return inner
    return decorator


"""
Warnings used in DewLoosh projects.
"""

class PerformanceWarning(Warning):
    
    def __init__(self, message: str):
        pre = "DewLoosh Performance Warning: "
        self.message = pre + message


# -*- coding: utf-8 -*-
import unittest

from dewloosh.core import Infix


class TestInfix(unittest.TestCase):

    def test_infix(self):
        x = Infix(lambda x, y: x * y)
        self.assertEqual(2 | x | 4, 8)
        x = Infix(lambda x, y: x + y)
        self.assertEqual(2 << x >> 4, 6)


if __name__ == "__main__":
    
    unittest.main()


# -*- coding: utf-8 -*-
import unittest

from dewloosh.core import attributor


class TestAttributor(unittest.TestCase):

    def test_attributor(self):
        axiom = attributor('__isaxiom__')
        abstractaxiom = attributor('__isaxiom__', '__isabstractmethod__')

        @axiom
        def foo(a, b): return 'an axiom'
        self.assertTrue(foo.__isaxiom__)
        
        @abstractaxiom
        def foo(a, b): return 'an abstract axiom'
        self.assertTrue(foo.__isaxiom__)
        self.assertTrue(foo.__isabstractmethod__)

if __name__ == "__main__":
    
    unittest.main()


# -*- coding: utf-8 -*-
try:
    from collections.abc import Iterable
except ImportError:
    from collections import Iterable
from typing import Callable


def isinkwargs(keys, **kwargs):
    if isinstance(keys, Iterable) and not isinstance(keys, str):
        return [key in kwargs for key in keys]
    else:
        return keys in kwargs


def allinkwargs(keys, **kwargs):
    if isinstance(keys, Iterable) and not isinstance(keys, str):
        return all([key in kwargs for key in keys])
    else:
        return keys in kwargs


def anyinkwargs(keys, **kwargs):
    if isinstance(keys, Iterable) and not isinstance(keys, str):
        return any([key in kwargs for key in keys])
    else:
        return keys in kwargs


def getfromkwargs(keys, default=None, astype=None, **kwargs):
    res = [kwargs.get(k, default) for k in keys]
    if astype is None:
        return res
    else:
        return [astype(p) for p in res]


def popfromkwargs(keys, d: dict = None, *args, default=None, astype=None,
                **kwargs):
    res = [d.pop(k, default) for k in keys]
    if astype is None:
        return res
    else:
        return [astype(p) for p in res]


def getallfromkwargs(keys, default=None, **kwargs):
    params = getfromkwargs(keys, default=default, **kwargs)
    if None not in params:
        return params
    else:
        missing = list(filter(lambda p: p == default, params))
        if len(missing) == 1:
            key = keys[missing[0]]
            raise RuntimeError("Parameter {} is missing from the definition!"
                               .format(key))
        else:
            missing_keys = [keys[i] for i in missing]
            raise RuntimeError("Parameters {} is missing from the definition!"
                               .format(missing_keys))


def getasany(keys, default=None, **kwargs):
    try:
        condition = [key in kwargs for key in keys]
        if not any(condition) == True:
            return default
        return kwargs[keys[condition.index(True)]]
    except Exception:
        return None
    

def countkwargs(fnc: Callable, **kwargs):
    assert callable(fnc)
    return sum(list(map(fnc, kwargs.keys())))




# -*- coding: utf-8 -*-
import unittest

from dewloosh.core.wrapping import Wrapper, customwrapper, wrap, wrapper


class TestWrap(unittest.TestCase):

    def test_wrap(self):
        class Wrapped:

            def foo(self):
                print('foo in object to be wrapped')

            def boo(self):
                print('boo in object to be wrapped')

        @customwrapper(wrapkey='wrapkey', wraptype=Wrapped)
        class CustomWrapper:

            def boo(self):
                print('boo in custom wrapper')

        @wrapper
        class DefaultWrapper:

            def boo(self):
                print('boo in default wrapper')

        class DefaultWrapper2(Wrapper):

            def boo(self):
                print('boo in default wrapper')

        cw = CustomWrapper(wrapkey=Wrapped())
        cw.foo()
        cw.boo()

        dw = DefaultWrapper(wrap=Wrapped())
        dw.foo()
        dw.boo()

        dw2 = DefaultWrapper(wrap=Wrapped())
        dw2.foo()
        dw2.boo()

        w = wrap(Wrapped())
        w.foo()
        w.boo()
        
        w.wrapped
        w.wraps()
        w.wrapped_obj()

        obj = CustomWrapper('a', Wrapped())
        
        @customwrapper(wrapkey='wrap', wraptype=dict)
        class CustomWrapper:

            def boo(self):
                print('boo in custom wrapper')
                
        obj = CustomWrapper('a', dict(a=2))
        assert obj['a'] == 2
        
        obj = CustomWrapper(a=2)
        assert obj['a'] == 2
        
if __name__ == "__main__":
    
    unittest.main()
